/***************************************************************************
                          ctransfer.h  -  description
                             -------------------
    begin                : Fri Feb 22 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTRANSFER_H
#define CTRANSFER_H

#include <dclib/dcos.h>
#include <dclib/cstring.h>
#include <dclib/cconnection.h>
#include <dclib/cbytearray.h>

/**
  *@author Mathias Kster
  */

class DLL_EXPORT CTransfer : public CConnection {
public:
	CTransfer();
	~CTransfer();

	/** callback function */
	virtual int DC_CallBack( CObject * ) { return -1; };
	/** */
	virtual void DataAvailable( const char * buffer, int len );
	/** */
	virtual void DataSend();
	/** */
	virtual void ConnectionState( eConnectionState state );

	/** */
	int StartDownload( CString dstfile, long pos, CString srcfile );
	/** */
	int StartUpload( CString dstfile, long length, long pos, CString srcfile );

	/** */
	unsigned long long GetTransferrate();
	/** */
	unsigned long long GetBytesForTransferrate( unsigned long long rate );

	/** */
	int HandleMessage( char * c, int len );

	/** */
	CString GetDstFilename() const;
	/** */
	void SetDstFilename( CString filename );
	/** */
	void SetSrcFilename( CString filename );
	/** */
	CString GetDstNick() const;
	/** */
	CString GetHubHost() const;
	/** */
	void SetHubHost( CString hubhost );
	/** */
	CString GetHubName() const;
	/** */
	void SetHubName( CString hubname );
	/** */
	eltMedium GetMedium() const;
	/** */
	void SetMedium( eltMedium medium );
	/** */
	long GetBuffer( CByteArray *ba );
	/** */
	void SetBuffer( CByteArray *ba );
	/** */
	eDirection GetSrcDirection() const;
	/** */
	void SetSrcDirection( eDirection direction );
	/** */
	int GetLevel() const;
	/** */
	eConnectionState GetMode() const;
	/** */
	void SetMode( eConnectionState mode );
	/** */
	unsigned long long GetPos() const;
	/** */
	void SetPos( unsigned long long pos );
	/** */
	unsigned long long GetLength() const;
	/** */
	void SetLength( unsigned long long length );
	/** */
	unsigned long long GetTransfered() const;
	/** */
	void InitStartTime();
	/** */
	void SetNick( CString nick );
	/** */
	void SetRate( unsigned long long rate );
	/** */
	bool IsIdle() const;
private:
	/** */
	CString sNick;
	/** */
	CString sBuffer;
	/** */
	unsigned long long lLength;
	/** */
	unsigned long long lTransfered;
	/** */
	unsigned long long lPos;
	/** */
	long lBufferPos;
	/** */
	unsigned long long lRate;
	/** */
	eConnectionState eMode;
	/** */
	eltMedium eMedium;
	/** */
	bool bIdle;
	/** */
	int fd;
	/** */
	CByteArray * pByteArray;
	/** local filename */
	CString sSrcFilename;
	/** remote filename */
	CString sDstFilename;
	/** local direction */
	eDirection eSrcDirection;
	/** remote direction */
	eDirection eDstDirection;
	/** remote nick */
	CString sDstNick;
	/** local level */
	int iLevel;
	/** identifier hubname */
	CString sHubName;
	/** identifier hubhost:hubport */
	CString sHubHost;
	/** */
	time_t starttime;
	/** */
	time_t currtime;
	/** */
	CThread TransferThread;
};

/** */
inline CString CTransfer::GetDstFilename() const
{ return sDstFilename; };
/** */
inline void CTransfer::SetDstFilename( CString filename )
{ TransferThread.Lock(); sDstFilename=filename; TransferThread.UnLock(); };
/** */
inline void CTransfer::SetSrcFilename( CString filename )
{ TransferThread.Lock(); sSrcFilename=filename; TransferThread.UnLock(); };
/** */
inline CString CTransfer::GetDstNick() const
{ return sDstNick; };
/** */
inline CString CTransfer::GetHubHost() const
{ return sHubHost; };
/** */
inline void CTransfer::SetHubHost( CString hubhost )
{ TransferThread.Lock(); sHubHost=hubhost; TransferThread.UnLock(); };
/** */
inline CString CTransfer::GetHubName() const
{ return sHubName; };
/** */
inline void CTransfer::SetHubName( CString hubname )
{ TransferThread.Lock(); sHubName=hubname; TransferThread.UnLock(); };
/** */
inline void CTransfer::SetMedium( eltMedium medium )
{ TransferThread.Lock(); eMedium=medium; TransferThread.UnLock(); };
/** */
inline eltMedium CTransfer::GetMedium() const
{ return eMedium; };
/** */
inline void CTransfer::SetBuffer( CByteArray *ba )
{ TransferThread.Lock(); pByteArray->Append(ba->Data(),ba->Size()); TransferThread.UnLock(); };
/** */
inline long CTransfer::GetBuffer( CByteArray *ba )
{ TransferThread.Lock(); long i; ba->Append(pByteArray->Data(),pByteArray->Size()); i = pByteArray->Size(); TransferThread.UnLock(); return i; };
/** */
inline eDirection CTransfer::GetSrcDirection() const
{ return eSrcDirection; };
/** */
inline void CTransfer::SetSrcDirection( eDirection direction )
{ TransferThread.Lock(); eSrcDirection = direction; TransferThread.UnLock(); };
/** */
inline int CTransfer::GetLevel() const
{ return iLevel; };
/** */
inline eConnectionState CTransfer::GetMode() const
{ return eMode; };
/** */
inline void CTransfer::SetMode( eConnectionState mode )
{ TransferThread.Lock(); eMode = mode; TransferThread.UnLock(); };
/** */
inline unsigned long long CTransfer::GetPos() const
{ return lPos; };
/** */
inline void CTransfer::SetPos( unsigned long long pos )
{ TransferThread.Lock(); lPos=pos; TransferThread.UnLock(); };
/** */
inline unsigned long long CTransfer::GetLength() const
{ return lLength; };
/** */
inline void CTransfer::SetLength( unsigned long long length )
{ TransferThread.Lock(); lLength = length; TransferThread.UnLock(); };
/** */
inline unsigned long long CTransfer::GetTransfered() const
{ return lTransfered; };
/** */
inline void CTransfer::InitStartTime()
{  TransferThread.Lock(); starttime = time(0);  TransferThread.UnLock(); };
/** */
inline void CTransfer::SetNick(CString nick)
{ TransferThread.Lock(); sNick=nick; TransferThread.UnLock(); };
/** */
inline void CTransfer::SetRate( unsigned long long rate )
{ TransferThread.Lock(); lRate=rate; TransferThread.UnLock(); };
/** */
inline bool CTransfer::IsIdle() const
{ return bIdle; };


#endif
