/***************************************************************************
                          dcobject.h  -  description
                             -------------------
    begin                : Fri Feb 22 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dclib/cstring.h>
#include <dclib/cobject.h>
#include <dclib/clist.h>

#ifndef _DCOBJECT_H_
#define _DCOBJECT_H_

typedef enum eDCMessage {
	DC_MESSAGE_PARSE_ERROR=0,

	DC_MESSAGE_CONNECTION_STATE,	// internal con. state
	DC_MESSAGE_TRANSFER,		// transfer state

	DC_MESSAGE_UNKNOWN,
	DC_MESSAGE_LOCK,
	DC_MESSAGE_HELLO,
	DC_MESSAGE_HUBNAME,
	DC_MESSAGE_CHAT,
	DC_MESSAGE_MYINFO,
	DC_MESSAGE_QUIT,
	DC_MESSAGE_NICKLIST,		// 10
	DC_MESSAGE_OPLIST,
	DC_MESSAGE_CONNECTTOME,
	DC_MESSAGE_KEY,
	DC_MESSAGE_PRIVATECHAT,
	DC_MESSAGE_SEARCH,
	DC_MESSAGE_REVCONNECTTOME,
	DC_MESSAGE_SEARCHRESULT,
	DC_MESSAGE_FORCEMOVE,
	DC_MESSAGE_MYNICK,		// connect to me
	DC_MESSAGE_DIRECTION,		// 20 connect to me
	DC_MESSAGE_MAXEDOUT,
	DC_MESSAGE_FILELENGTH,
	DC_MESSAGE_ERROR,		// file not found ...
	DC_MESSAGE_GET,
	DC_MESSAGE_GETLISTLEN,
	DC_MESSAGE_VALIDATEDENIDE,
	DC_MESSAGE_GETINFO,
	DC_MESSAGE_AGE,
	DC_MESSAGE_HUBISFULL,
	DC_MESSAGE_LISTLEN,		// 30
	DC_MESSAGE_SEND,
	DC_MESSAGE_PING,
	DC_MESSAGE_GETPASS,		// user need a password
	DC_MESSAGE_BADPASS,		// wrong password
	DC_MESSAGE_LOGEDIN		// admin
};

/** transfer medium */
typedef enum eltMedium {
	eltNONE=0,
	eltBUFFER,
	eltFILE
};

/** direction */
typedef enum eDirection {
	edNONE=0,
	edUPLOAD,
	edDOWNLOAD
} eDirection;

/** socket state */
typedef enum eConnectionState {
	estNONE=0,
	estCONNECT,
	estCONNECTING,
	estCONNECTED,
	estDISCONNECTING,
	estDISCONNECTED,
	estSOCKETERROR,
	estCONNECTIONTIMEOUT,
	estTRANSFERHANDSHAKE,
	estTRANSFERDOWNLOAD,
	estTRANSFERUPLOAD
};

class DLL_EXPORT CDCMessage : public CObject {
public:
	eDCMessage eType;
};

class DLL_EXPORT CMessagePing : public CDCMessage {
public:
};

class DLL_EXPORT CMessageGetInfo : public CDCMessage {
public:
	CString sNick;
	CString sFrom;
};

class DLL_EXPORT CMessageConnectionState : public CDCMessage {
public:
	eConnectionState eState;
	CString sMessage;
};

class DLL_EXPORT CMessageTransfer : public CDCMessage {
public:
	int iMode;		/* 0: send 1:recv */
	unsigned long long lTransfered;
	unsigned long long lLength;
};

class DLL_EXPORT CMessageError : public CDCMessage {
public:
	CString sError;
};

class DLL_EXPORT CMessageGet : public CDCMessage {
public:
	CString sFilename;
	unsigned long long lPos;
};

class DLL_EXPORT CMessageGetPass : public CDCMessage {
public:
};

class DLL_EXPORT CMessageBadPass : public CDCMessage {
public:
};

class DLL_EXPORT CMessageLogedIn : public CDCMessage {
public:
	CString sNick;
};

class DLL_EXPORT CMessageSend : public CDCMessage {
public:
};

class DLL_EXPORT CMessageValidateDenide : public CDCMessage {
public:
};

class DLL_EXPORT CMessageHubIsFull : public CDCMessage {
public:
};

class DLL_EXPORT CMessageMaxedOut : public CDCMessage {
public:
};

class DLL_EXPORT CMessageGetListLen : public CDCMessage {
public:
};

class DLL_EXPORT CMessageLock : public CDCMessage {
public:
	CString sPK;
	CString sData;
};

class DLL_EXPORT CMessageFileLength : public CDCMessage {
public:
	unsigned long long lFileLength;
};

class DLL_EXPORT CMessageDirection : public CDCMessage {
public:
	eDirection Direction;
	int iLevel;
};

class DLL_EXPORT CMessageNickList : public CDCMessage {
public:
	CList<CObject> NickList;
};

class DLL_EXPORT CMessageOpList : public CDCMessage {
public:
	CList<CObject> OpList;
};

class DLL_EXPORT CMessageNick : public CDCMessage {
public:
	CString sNick;
};

class DLL_EXPORT CMessageMyNick : public CDCMessage {
public:
	CString sMyNick;
};

class DLL_EXPORT CMessageHello : public CDCMessage {
public:
	CString sNick;
};

class DLL_EXPORT CMessageQuit : public CDCMessage {
public:
	CString sNick;
};

class DLL_EXPORT CMessageChat : public CDCMessage {
public:
	CString sNick;
	CString sMessage;
};

class DLL_EXPORT CMessageMyInfo : public CDCMessage {
public:
	CString sNick;
	CString sComment;
	CString sUnknown;
	CString sSpeed;
	CString sEMail;
	unsigned long long lShared;
	int iAwayMode;
};

class DLL_EXPORT CMessageConnectToMe : public CDCMessage {
public:
	CString sNick;
	CString sHost;
	int 	iPort;
};

class DLL_EXPORT CMessageRevConnectToMe : public CDCMessage {
public:
	CString sNick;
	CString sRemoteNick;
};

class DLL_EXPORT CMessageSearch : public CDCMessage {
public:
	CString sNick;
	CString sFlag;
	CString sString;
};

class DLL_EXPORT CMessageForceMove : public CDCMessage {
public:
	CString sHost;
	int	iPort;
};

class DLL_EXPORT CMessageSearchResult : public CDCMessage {
public:
	CString sNick;
	CString sFile;
	unsigned long long lSize;
	CString sSlot;
	CString sHubName;
	CString sHubHost;
};

class DLL_EXPORT CMessagePrivateChat : public CDCMessage {
public:
	CString sNick;
	CString sFrom;
	CString sString;
};

class DLL_EXPORT CMessageHubName : public CDCMessage {
public:
	CString sHubName;
};

#endif
